// Copyright - Simovits Consulting 2009
// Author: James Dickson.

#ifndef _common_h
#define _common_h

#include <winsock2.h>
#include <direct.h>
#include <string>
#include <openssl/err.h>

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <openssl/ssl.h>

extern BIO *bio_err;

#define RECEIVE_FAIL 0
#define CONNECT_FAIL 0
#define CONNECT_OK	1
#define SEND_OK		2
#define RECEIVE_OK	4

#define CERT_OK				0
#define CERT_ERROR_NOT_VERIFIED	1
#define CERT_ERROR_CN_MISMATCH	2	


enum RequestProtocol
{
	SSL2,
	SSL3,
	TLS1,
	SSL2OR3,
	DTLS1		// 2010-03-24 - james - Must add this.
};


/**
 * 2010-07-07 
 */
static struct CRYPTO_INFO
{
	int bits;

	char strOpenSSLName[256];
	char strOpenSSLShortName[50];
	char strStrengthName[50];
	int strength;


}*LPCRYPTO_INFO;


typedef struct tagCIPHERTESTITEM
{
	RequestProtocol protocol;
	int testCount;
	char *strVersion;

	CRYPTO_INFO cinf;		

}CIPHERTESTITEM, *LPCIPHERTESTITEM;

// Structs
typedef struct tagTESTINFO
{
	char *strHostname;
	char *strIPAdress;
	char *strUseragent;
	char *strGetHostname;
	char *strCACertPath;
	char *strRequest;
	char *strSIPDomain;
	char *strServerCipher;
	char *strServerCertPath;
	char *strServerCertPassword;
	char *strServerCertKeyfile;
	char *strHTTPPage;

	BOOL testSIP;
	BOOL bPrintCommunication;
	BOOL bTestOnlySpecificCryptos;
	BOOL bTestOnlySpecificProtocols;
	BOOL bServerMode;

	int port;
	int debugLevel;
	int secondTimeout;
	int msInterval;
	int onlyGrade;
	BOOL bTestSend;
	BOOL bVerbose;
	BOOL bSuppressSummary;
	BOOL bCheckCert;
	//BOOL bSSLRatingGuide; // 2010-07-07

	BOOL bUDP;		// 2010-03-24 - Use UDP instead of TCP (used for DTLS)

	RequestProtocol serverProtocol;

	BOOL bExitOnBitMismatch;
	BOOL bUseOpenSSLBits;	
	
	BOOL bPrintAvailableCiphers;
	BOOL bOnlyAvailableCiphers;
	BOOL bPrintCert;

	BOOL bSuppressUserSupplied;
	BOOL bStartTLS; 
	char strSMTPConnect[1024];

	BOOL bUseWorkarounds;
	BOOL bUseNessusCipherTest;
	BOOL bTestRenegotiation;

	BOOL bUseColors;

	BOOL bUseCiscoSpeshul;

} TESTINFO, *LPTESTINFO;

typedef struct tagCERTINFO
{
	X509 *certficate;
	int verResult;
}CERTINFO;



// Used when a new successful connection is found.
typedef struct tagFOUNDSUCCESS
{
	BOOL bConnect;
	BOOL bSend;
	BOOL bReceive;
	BOOL bRenegotiate;
	BOOL bSafeRenegotiate;
	
	CIPHERTESTITEM cinfo;
	CERTINFO certInfo;

}FOUNDSUCCESS;


int printCiphersForMethod(RequestProtocol protocol);
int isInt(char *str, int len);
int getCertBitCount(X509 *x509Cert);
int getCertTypeString(int type, char *buf);
void printUsage();
int findCryptoIndex(char *strCrypto);
void initTestInfo(TESTINFO *tInf);
int getAvailableCiphers(CIPHERTESTITEM *cipherItems, int maxItems, TESTINFO testInfo);
void printCert(X509 *crtCurrent);

SSL_CTX *initCTX(RequestProtocol protocol, BOOL bUseWorkarounds, BOOL bUseCiscoSpeshul);
BOOL getCertAlgortithm(X509 *crtCurrent, char *buf, int buflen);
void strtoupper(char *str, int maxlen);

#endif


